## Script to image the B2B B4 to B9
## HL Tau testing data 
## NOTE this is not an SV dataset and
## observations were conducted in marginal conditions
## with a testing scheduling block setup

## This is the updated script and uses 'tclean'
## for compatability with CASA 5.6.1-8 - In the paper
## 'clean' was used in CASA 4.7.2 as the orignal data were
## taken in 2017. The imaging is a simple
## mfs image without multiscale cleaning
## a shallow clean and circular mask are the default

### Hardcoded script Final Version
### Luke Maud ESO & Yoshiharu Asaki JAO 
### For further information please contact:
### lmaud@eso.org   ESO
### yasaki@alma.cl   JAO/NAOJ
###
### If using these data please reference:
### Asaki et al. 2020 ApJS 247 23


## 2018-2019  - CASA 4.7.2 'clean'
## 24/02/2020 - CASA 5.6.1-8 'tclean' -


thesteps = []
step_title = {0: 'Combine the SBs B2B B4-B9 data',
              1: 'Image the Continuum - natural',
              2: 'Image the Continuum - natural >500kl',
              3: 'Image the Continuum - robust 0.5',
              4: 'Export to FITS format'
            }



try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = [0,1,2,3,4] 


import re
import os
import casadef

 
if re.search('^5.6.1', casadef.casa_version) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 5.6.1')


# concat and split the target 
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
 
  concat(vis=['uid___A002_Xc660ef_X1b89.ms.split.cal','uid___A002_Xc660ef_X2046.ms.split.cal'], concatvis='Combined_B2B_B4B9.ms.split.cal')
  split(vis='Combined_B2B_B4B9.ms.split.cal', outputvis='HLTAU_B2B_B4B9.ms.split.cal', datacolumn='data', field='HL Tau')

## Image - Natural
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  os.system('rm -rf HLTau_sci.spw17_19_21_23_25_27_29_31.cont.I.manual-natural.*')
  tclean(vis='HLTAU_B2B_B4B9.ms.split.cal', 
         imagename='HLTau_sci.spw17_19_21_23_25_27_29_31.cont.I.manual-natural', 
         field='HL Tau',  
         specmode='cont',
         outframe='LSRK',
         spw='', 
         imsize=2160,
         cell='0.003arcsec', 
         mask='circle[[1080pix,1080pix],120pix]',
         deconvolver='hogbom',
         weighting='natural',
         pbcor=True,
         phasecenter='J2000 04h31m38.428 +18d13m57.186', 
         interactive=False,
         niter=600,
         threshold='0.01mJy')


## image with UV limit 
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf HLTau_sci.spw17_19_21_23_25_27_29_31.cont.I.manual-natural500kl.*')
  tclean(vis='HLTAU_B2B_B4B9.ms.split.cal', 
         imagename='HLTau_sci.spw17_19_21_23_25_27_29_31.cont.I.manual-natural500kl', 
         field='HL Tau',  
         specmode='cont',
         outframe='LSRK',
         spw='', 
         imsize=2160,
         cell='0.003arcsec', 
         mask='circle[[1080pix,1080pix],120pix]',
         deconvolver='hogbom',
         weighting='natural',
         pbcor=True,
         phasecenter='J2000 04h31m38.428 +18d13m57.186', 
         interactive=False,
         niter=600,
         uvrange='>500klambda',
         threshold='0.01mJy')


 


## image with robust 
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf HLTau_sci.spw17_19_21_23_25_27_29_31.cont.I.manual-robust.*')
  tclean(vis='HLTAU_B2B_B4B9.ms.split.cal', 
         imagename='HLTau_sci.spw17_19_21_23_25_27_29_31.cont.I.manual-robust', 
         field='HL Tau',  
         specmode='cont',
         outframe='LSRK',
         spw='', 
         imsize=2160,
         cell='0.003arcsec', 
         mask='circle[[1080pix,1080pix],120pix]',
         deconvolver='hogbom',
         weighting='briggs',
         robust=0.5,
         pbcor=True,
         phasecenter='J2000 04h31m38.428 +18d13m57.186', 
         interactive=False,
         niter=600,
         threshold='0.01mJy')


## Export the FITS 
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  myimages = ['HLTau_sci.spw17_19_21_23_25_27_29_31.cont.I.manual-natural',
              'HLTau_sci.spw17_19_21_23_25_27_29_31.cont.I.manual-natural500kl',
              'HLTau_sci.spw17_19_21_23_25_27_29_31.cont.I.manual-robust']
  
  for myimagebase in myimages:
    exportfits(imagename = myimagebase+'.image.pbcor',
               fitsimage = myimagebase+'.pbcor.fits',
               overwrite = True
               )
    if os.path.exists(myimagebase+'.pb'):
      exportfits(imagename = myimagebase+'.pb',
                 fitsimage = myimagebase+'.pb.fits',
                 overwrite = True
                 )
  

