# ALMA Data Reduction Script

### This is a QA2-like script for calibration for the Band-to-Band
### testing dataset for HL-Tau observed at band 9 during the 2017 early
### Cycle 5 SB testing for the High-Frequency Long Baselines EOC campaign,
### Here, the B2B uses phase transfer from a phase calibrator observed at band 4. 

### This is NOT a Science Verification dataset - these are test data, taken
### in marginal conditions, and that had a few issues in aquesition and the 
### reduction due to this testing nature. E.g. the DGC sequence did not 
### perform the loop as required - it is too short for finding a good
### B2B offet

### Notes will be also added during the steps to help/inform inquisitive
### users. Note also, these data were taken with SBs but specifically for
### testing purposes. These data were not, and are not science verification
### datasets and thus do not fully follow a 'standard' observational sequence.

### For further information please contact:
### lmaud@eso.org   ESO
### yasaki@alma.cl   JAO/NAOJ
###
### If using these data please reference:
### Asaki et al 2020 ApJS 247 23A
### and B2B papers therein to be published 

### Author Luke Maud ESO
### Hardcoded script Final Version
### Luke Maud ESO & Yoshiharu Asaki JAO 
### 16/07/2019 - CASA 5.4.0
### 10/03/2020 - CASA 5.6.1-8 version


thesteps = []
step_title = {0: 'Import of the ASDMs',
              1: 'Fix of SYSCAL table times',
              2: 'listobs',
              3: 'A priori flagging',
              4: 'Generation and time averaging of the WVR cal table',
              5: 'Generation of the Tsys cal table',
              6: 'Generation of the antenna position cal table',
              7: 'Application of the WVR, Tsys and antpos cal tables',
              8: 'Split out science SPWs and time average',
	      9: 'Listobs after split',
	      10: 'Initial flagging',
              11: 'Putting a model for the flux calibrator(s)',
              12: 'Save flags before bandpass cal',
	      13: 'Bandpass Calibration',
              14: 'Save flags before Gain calibration',
	      15: 'Gain Calibration - DGC',
	      16: 'Gain Calibration - Phase and Amp cal',
	      17: 'Save flags before applycal',
	      18: 'Application of the bandpass and gain cal tables',
              19: 'Split out corrected column',
              20: 'Save flags after applycal'}

if 'applyonly' not in globals(): applyonly = True
try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]
# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

import casadef

if applyonly != True: es = aU.stuffForScienceDataReduction() 


if re.search('^5.6.1', '.'.join([str(i) for i in cu.version().tolist()[:-1]])) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 5.6.1')


# CALIBRATE_AMPLI: 
# CALIBRATE_ATMOSPHERE: 
# CALIBRATE_BANDPASS: J0522-3627
# CALIBRATE_DIFFGAIN: J0522-3627
# CALIBRATE_FLUX: J0522-3627
# CALIBRATE_FOCUS: 
# CALIBRATE_PHASE: J0431+1731
# CALIBRATE_POINTING: J0449+1121
# CALIBRATE_POLARIZATION: 
# OBSERVE_CHECK: J0431+2037
# OBSERVE_TARGET: HL Tau


# Using reference antenna = DV07

# Import of the ASDM
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  ## this step will always remove the MS - as re-running the complex conjugate correction
  ## on a pre-exiting MS will incorrect change the phases again

  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms')
  os.system('mv uid___A002_Xc660ef_X1b89.asdm.sdm uid___A002_Xc660ef_X1b89')
  if os.path.exists('uid___A002_Xc660ef_X1b89.ms') == False:
    importasdm('uid___A002_Xc660ef_X1b89', asis='*', bdfflags=True, lazy=False, process_caldevice=False)
    if not os.path.exists('uid___A002_Xc660ef_X1b89.ms.flagversions'):
      print 'ERROR in importasdm. Output MS is probably not useful. Will stop here.'
      thesteps = []
  if applyonly != True: es.fixForCSV2555('uid___A002_Xc660ef_X1b89.ms')


  ## unavoidable correction for these test data
  ## Walsh switching enabled in the Cycle 5 B2B software recorded the LF as
  ## the complex conjugate of the 'true' data 
  ## i.e the phases were reverese thus subsequence phase calibration would not work without any correction
  tb.open('uid___A002_Xc660ef_X1b89.ms', nomodify=False)
  for desc_id in range(32,50):  # spwLowInt:
          raw_data = tb.query('DATA_DESC_ID==%s' % desc_id)  
          uncal_data = raw_data.getcol('DATA')
          raw_data.putcol('DATA', uncal_data.conj())
          raw_data.close()
  tb.close()



# Fix of SYSCAL table times
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  from recipes.almahelpers import fixsyscaltimes
  fixsyscaltimes(vis = 'uid___A002_Xc660ef_X1b89.ms')




print "# A priori calibration"

# listobs & antpos 
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.listobs')
  listobs(vis = 'uid___A002_Xc660ef_X1b89.ms',
    listfile = 'uid___A002_Xc660ef_X1b89.ms.listobs')
  

  plotants(vis='uid___A002_Xc660ef_X1b89.ms', 
    figfile='uid___A002_Xc660ef_X1b89.ms.antpos.png')

  # Extra note - there will be 8 LF spectral windows, this is a 'feature' of the testing B2B
  # mode when walsh switching is employed (to give 2x BW at the HF). Here only 4 of the 
  # LF SPWs are actaully containing real data 
  

# A priori flagging
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagdata(vis = 'uid___A002_Xc660ef_X1b89.ms', 
      mode = 'manual', autocorr = True, flagbackup = False)

  flagdata(vis = 'uid___A002_Xc660ef_X1b89.ms',
      mode = 'manual',  intent = '*POINTING*,*ATMOSPHERE*', flagbackup = False)



  flagcmd(vis = 'uid___A002_Xc660ef_X1b89.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'plot',
    plotfile = 'uid___A002_Xc660ef_X1b89.ms.flagcmd.png')
  
  flagcmd(vis = 'uid___A002_Xc660ef_X1b89.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'apply')


# Generation and time averaging of the WVR cal table
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.wvr')   
  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.wvrgcal') 
  


  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_Xc660ef_X1b89.ms.wvrgcal')
  
  wvrgcal(vis = 'uid___A002_Xc660ef_X1b89.ms', 
      caltable = 'uid___A002_Xc660ef_X1b89.ms.wvr') 
 

  casalog.setlogfile(mylogfile)


  # Extra Notes
  # This makes the WVR solutions plots for both frequencies,
  # those for the higher frequency SHOULD look to have a larger phase variations (band 4 to band 9, Hf/Lf is more than a factor of 4)
  # as phase variations scales linearly with frequency  
  
  if applyonly != True: aU.plotWVRSolutions(caltable='uid___A002_Xc660ef_X1b89.ms.wvr', spw='38', antenna='DV07', yrange=[-199,199],subplot=22, interactive=False, figfile='uid___A002_Xc660ef_X1b89.ms.wvr.low.plots/uid___A002_Xc660ef_X1b89.ms.wvr') 

  if applyonly != True: aU.plotWVRSolutions(caltable='uid___A002_Xc660ef_X1b89.ms.wvr', spw='17', antenna='DV07', yrange=[-199,199],subplot=22, interactive=False, figfile='uid___A002_Xc660ef_X1b89.ms.wvr.high.plots/uid___A002_Xc660ef_X1b89.ms.wvr') 

  #Note: If you see wraps in these plots, try changing yrange or unwrap=True 
  #Note: If all plots look strange, it may be a bad WVR on the reference antenna.
  #      To check, you can set antenna='' to show all baselines.
  



# Generation of the Tsys cal table
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.tsys') 
  gencal(vis = 'uid___A002_Xc660ef_X1b89.ms',
	 caltable = 'uid___A002_Xc660ef_X1b89.ms.tsys',
	 caltype = 'tsys')

  # Flagging edge channels of Tsys

  # Extra notes - 8 HF SPWs, 4 'real' LF SPWs + 4 Redundant LF SPW
  # due to how the data and Tsys are saved - some get data, some get Tsys
  # but are the 'same' SPW but 'mirrored' due to the Walsh Switching effecting
  # the LF SPWs 
 
 
  flagdata(vis = 'uid___A002_Xc660ef_X1b89.ms.tsys',
	   mode = 'manual',
	   spw = '7:0~7;120~127,17:0~7;120~127,19:0~7;120~127,21:0~7;120~127,23:0~7;120~127,25:0~7;120~127,27:0~7;120~127,29:0~7;120~127,31:0~7;120~127,36:0~7;120~127,38:0~7;120~127,41:0~7;120~127,43:0~7;120~127,45:0~7;120~127,47:0~7;120~127,49:0~7;120~127',  
	   flagbackup = False)
  
  ## Unavoidable to open the Tsys tables and copy the matching 'pair' 
  ## Tsys to the 'empty' Tsys SPWs, as these contain the 'real' data
  ## and allows later SPW Tsys mapping to work correctly

  tb.open('uid___A002_Xc660ef_X1b89.ms.tsys', nomodify=False)

  Tsys7Data = tb.query('SPECTRAL_WINDOW_ID==7')
  Tsys7 = Tsys7Data.getcol('FPARAM')
  Tsys41Data = tb.query('SPECTRAL_WINDOW_ID==41')
  Tsys41Data.putcol('FPARAM', Tsys7)
          
  Tsys36Data = tb.query('SPECTRAL_WINDOW_ID==36')
  Tsys36 = Tsys36Data.getcol('FPARAM')
  Tsys38Data = tb.query('SPECTRAL_WINDOW_ID==38')
  Tsys38Data.putcol('FPARAM', Tsys36)

  Tsys43Data = tb.query('SPECTRAL_WINDOW_ID==43')
  Tsys43 = Tsys43Data.getcol('FPARAM')
  Tsys45Data = tb.query('SPECTRAL_WINDOW_ID==45')
  Tsys45Data.putcol('FPARAM', Tsys43)

  Tsys47Data = tb.query('SPECTRAL_WINDOW_ID==47')
  Tsys47 = Tsys47Data.getcol('FPARAM')
  Tsys49Data = tb.query('SPECTRAL_WINDOW_ID==49')
  Tsys49Data.putcol('FPARAM', Tsys47)
  
  tb.close()
  

  if applyonly != True: aU.plotbandpass(caltable='uid___A002_Xc660ef_X1b89.ms.tsys', overlay='time', xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False, showatm=True,pwv='auto',chanrange='92.1875%',showfdm=True, showBasebandNumber=True, showimage=False, field='', figfile='uid___A002_Xc660ef_X1b89.ms.tsys.plots.overlayTime/uid___A002_Xc660ef_X1b89.ms.tsys') 
  
  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X1b89.ms.tsys', msName='uid___A002_Xc660ef_X1b89.ms', interactive=False)


# Generation of the antenna position cal table
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  
  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.antpos') 
  gencal(vis = 'uid___A002_Xc660ef_X1b89.ms',
    caltable = 'uid___A002_Xc660ef_X1b89.ms.antpos',
    caltype = 'antpos',
    antenna = 'DA41,DA43,DA44,DA45,DA46,DA47,DA48,DA49,DA50,DA51,DA52,DA53,DA54,DA55,DA56,DA57,DA58,DA59,DA60,DA63,DA64,DA65,DV01,DV04,DV07,DV08,DV10,DV11,DV12,DV13,DV14,DV15,DV17,DV19,DV22,DV23,DV24,DV25,PM01,PM02,PM03,PM04',
      #  parameter = [-1.78142e-05,8.30644e-04,4.10085e-04,-4.00916e-05,6.65741e-04,9.64524e-05,-5.13782e-04,8.87891e-04,-1.47149e-07,1.53058e-04,8.01244e-05,2.52155e-04,2.26063e-04,-5.81427e-04,-2.66012e-05,-2.25006e-05,5.91375e-04,6.45005e-04,6.83242e-04,-1.52784e-03,-4.22246e-04,3.30791e-05,3.94730e-04,4.06143e-04,2.11079e-04,2.67035e-04,5.99129e-04,-2.24102e-04,4.34216e-04,4.66974e-04,6.50448e-04,-9.34455e-04,-3.59355e-04,3.62997e-04,-1.46167e-03,-8.35765e-04,7.15067e-04,-3.41355e-03,-1.45851e-03,-1.83205e-04,6.40403e-04,2.95802e-04,-9.90769e-05,3.98024e-04,1.51837e-04,-1.72916e-04,7.66136e-04,5.05006e-04,1.34758e-04,9.41884e-05,5.59557e-04,3.13418e-05,-1.13010e-03,-9.20565e-04,4.06404e-04,3.42857e-04,-2.14315e-04,-3.06051e-04,-2.69569e-03,-6.40729e-04,-3.16965e-04,-7.83368e-05,2.37111e-04,1.41396e-04,-6.61986e-04,1.48773e-04,1.09947e-04,-2.76429e-04,-3.72016e-04,-2.38231e-04,-3.02599e-03,-3.10416e-04,1.29661e-04,-1.47888e-04,1.60294e-04,-5.98185e-04,-5.09675e-03,-2.33882e-03,4.00050e-04,4.65104e-04,5.80347e-04,-4.10370e-04,9.58987e-04,1.10625e-03,2.90854e-04,-8.97086e-04,-4.04840e-04,1.36811e-04,9.79351e-05,-6.51285e-04,-6.75720e-04,1.60965e-03,7.99060e-04,-1.01829e-03,1.96660e-03,4.95779e-04,1.31668e-05,5.12543e-04,4.97609e-04,-8.77078e-05,4.78648e-04,2.42355e-04,-1.84960e-03,1.08218e-03,2.98598e-04,-2.78929e-04,1.08367e-03,9.43055e-04,-2.06528e-04,1.42435e-03,1.16265e-03,5.01624e-05,2.10368e-04,7.30627e-05,-6.40727e-05,4.88757e-04,4.37836e-04,-9.17429e-05,3.21982e-04,2.89154e-04,-1.20766e-04,1.02105e-03,1.24334e-04,-6.39522e-04,7.31457e-04,-3.68585e-04] )# values 
    parameter = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]) 
# 0's no correction/update -->> images better without meaning the antenna positions in the asdm are fine
  
  
  # antenna x_offset y_offset z_offset total_offset baseline_date
  #DV08    -5.98185e-04   -5.09675e-03   -2.33882e-03    5.63957e-03      2017-10-06 03:03:50
  # DA54     7.15067e-04   -3.41355e-03   -1.45851e-03    3.78033e-03      2017-10-06 03:03:50
  # DV04    -2.38231e-04   -3.02599e-03   -3.10416e-04    3.05119e-03      2017-10-06 03:03:50
  # DA63    -3.06051e-04   -2.69569e-03   -6.40729e-04    2.78765e-03      2017-10-06 03:03:50
  # DV15    -1.01829e-03    1.96660e-03    4.95779e-04    2.26941e-03      2017-11-26 00:12:34
  # DV22    -1.84960e-03    1.08218e-03    2.98598e-04    2.16363e-03      2017-11-26 00:12:34
  # DV14    -6.75720e-04    1.60965e-03    7.99060e-04    1.91991e-03      2017-11-16 08:09:25
  # DV24    -2.06528e-04    1.42435e-03    1.16265e-03    1.85019e-03      2017-10-17 01:49:03
  # DA48     6.83242e-04   -1.52784e-03   -4.22246e-04    1.72610e-03      2017-10-06 03:03:50
  # DA53     3.62997e-04   -1.46167e-03   -8.35765e-04    1.72242e-03      2017-10-06 03:03:50
  # DV11    -4.10370e-04    9.58987e-04    1.10625e-03    1.52048e-03      2017-11-26 00:12:34
  # DV23    -2.78929e-04    1.08367e-03    9.43055e-04    1.46338e-03      2017-11-16 08:09:25
  # DA59     3.13418e-05   -1.13010e-03   -9.20565e-04    1.45792e-03      2017-10-06 03:03:50
  # DA52     6.50448e-04   -9.34455e-04   -3.59355e-04    1.19391e-03      2017-10-06 03:03:50
  # PM04    -6.39522e-04    7.31457e-04   -3.68585e-04    1.03917e-03      2017-11-26 00:12:34
  # PM03    -1.20766e-04    1.02105e-03    1.24334e-04    1.03566e-03      2017-11-16 08:09:25
  # DV12     2.90854e-04   -8.97086e-04   -4.04840e-04    1.02628e-03      2017-10-06 03:03:50
  # DA44    -5.13782e-04    8.87891e-04   -1.47149e-07    1.02583e-03      2017-11-26 00:12:34
  # DA57    -1.72916e-04    7.66136e-04    5.05006e-04    9.33754e-04      2017-11-16 08:09:25
  # DA41    -1.78142e-05    8.30644e-04    4.10085e-04    9.26529e-04      2017-11-26 00:12:34
  # DA47    -2.25006e-05    5.91375e-04    6.45005e-04    8.75364e-04      2017-11-16 08:09:25
  # DV10     4.00050e-04    4.65104e-04    5.80347e-04    8.44491e-04      2017-11-26 00:12:34
  # DA55    -1.83205e-04    6.40403e-04    2.95802e-04    7.28820e-04      2017-11-16 08:09:25
  # DV17     1.31668e-05    5.12543e-04    4.97609e-04    7.14484e-04      2017-11-16 08:09:25
  # DA65     1.41396e-04   -6.61986e-04    1.48773e-04    6.93074e-04      2017-11-16 08:09:25
  # DA50     2.11079e-04    2.67035e-04    5.99129e-04    6.89070e-04      2017-11-26 00:12:34
  # DA51    -2.24102e-04    4.34216e-04    4.66974e-04    6.75892e-04      2017-11-16 08:09:25
  # DA43    -4.00916e-05    6.65741e-04    9.64524e-05    6.73885e-04      2017-11-26 00:12:34
  # DV13     1.36811e-04    9.79351e-05   -6.51285e-04    6.72667e-04      2017-11-26 00:12:34
  # PM01    -6.40727e-05    4.88757e-04    4.37836e-04    6.59309e-04      2017-11-16 08:09:25
  # DA46     2.26063e-04   -5.81427e-04   -2.66012e-05    6.24396e-04      2017-11-16 08:09:25
  # DA58     1.34758e-04    9.41884e-05    5.59557e-04    5.83211e-04      2017-11-26 00:12:34
  # DA60     4.06404e-04    3.42857e-04   -2.14315e-04    5.73277e-04      2017-11-16 08:09:25
  # DA49     3.30791e-05    3.94730e-04    4.06143e-04    5.67325e-04      2017-11-16 08:09:25
  # DV19    -8.77078e-05    4.78648e-04    2.42355e-04    5.43629e-04      2017-11-16 08:09:25
  # DV01     1.09947e-04   -2.76429e-04   -3.72016e-04    4.76337e-04      2017-10-06 03:03:50
  # PM02    -9.17429e-05    3.21982e-04    2.89154e-04    4.42379e-04      2017-11-16 08:09:25
  # DA56    -9.90769e-05    3.98024e-04    1.51837e-04    4.37372e-04      2017-11-16 08:09:25
  # DA64    -3.16965e-04   -7.83368e-05    2.37111e-04    4.03516e-04      2017-11-16 08:09:25
  # DA45     1.53058e-04    8.01244e-05    2.52155e-04    3.05662e-04      2017-11-16 08:09:25
  # DV07     1.29661e-04   -1.47888e-04    1.60294e-04    2.53726e-04      2017-11-16 08:09:25
  # DV25     5.01624e-05    2.10368e-04    7.30627e-05    2.28274e-04      2017-11-16 08:09:25\n"
  


# Application of the WVR, Tsys and antpos cal tables
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  from recipes.almahelpers import tsysspwmap
  tsysmap = tsysspwmap(vis = 'uid___A002_Xc660ef_X1b89.ms', tsystable = 'uid___A002_Xc660ef_X1b89.ms.tsys', tsysChanTol = 1)



  # J0522-3627 has a tsys
  applycal(vis = 'uid___A002_Xc660ef_X1b89.ms',
    field = '0',
    spw = '7,17,19,21,23,25,27,29,31,36,38,41,43,45,47,49',
    gaintable = ['uid___A002_Xc660ef_X1b89.ms.tsys', 'uid___A002_Xc660ef_X1b89.ms.wvr', 'uid___A002_Xc660ef_X1b89.ms.antpos'],
    gainfield = ['0', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = True,
    flagbackup = False)

  
  # J0431+1731 has a tsys
  applycal(vis = 'uid___A002_Xc660ef_X1b89.ms',
    field = '2',
    spw = '7,17,19,21,23,25,27,29,31,36,38,41,43,45,47,49',
    gaintable = ['uid___A002_Xc660ef_X1b89.ms.tsys', 'uid___A002_Xc660ef_X1b89.ms.wvr', 'uid___A002_Xc660ef_X1b89.ms.antpos'],
    gainfield = ['2', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = True,
    flagbackup = False)


  # J0431+2037 does not have a tsys - use one from HLTau instead (becuase it is at the same frequency)
  applycal(vis = 'uid___A002_Xc660ef_X1b89.ms',
    field = '3',
    spw = '7,17,19,21,23,25,27,29,31,36,38,41,43,45,47,49',
    gaintable = ['uid___A002_Xc660ef_X1b89.ms.tsys', 'uid___A002_Xc660ef_X1b89.ms.wvr', 'uid___A002_Xc660ef_X1b89.ms.antpos'],
    gainfield = ['4', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = True,
    flagbackup = False)


  # HL Tau has a tsys
  applycal(vis = 'uid___A002_Xc660ef_X1b89.ms',
    field = '4',
    spw = '7,17,19,21,23,25,27,29,31,36,38,41,43,45,47,49',
    gaintable = ['uid___A002_Xc660ef_X1b89.ms.tsys', 'uid___A002_Xc660ef_X1b89.ms.wvr', 'uid___A002_Xc660ef_X1b89.ms.antpos'],
    gainfield = ['4', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = True,
    flagbackup = False)


  if applyonly != True: es.getCalWeightStats('uid___A002_Xc660ef_X1b89.ms')   



# Split out science SPWs and time average

mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Extra note:
  # This is the full split file - with corrected SPW orders this can
  # keep the high and low freq together, high 0,1,2,3,4,5,6,7 and low 8,9,10,11
  # The 'real' LF SPWs are 38,41,43,47 with data

  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.split') 
  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.split.flagversions') 
  split(vis = 'uid___A002_Xc660ef_X1b89.ms',
    outputvis = 'uid___A002_Xc660ef_X1b89.ms.split',
    datacolumn = 'corrected',
    spw = '17,19,21,23,25,27,29,31,38,41,43,47',  
    keepflags = True)




# Listobs, and save original flags
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.split.listobs')
  listobs(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
    listfile = 'uid___A002_Xc660ef_X1b89.ms.split.listobs')
  
  
  if not os.path.exists('uid___A002_Xc660ef_X1b89.ms.split.flagversions/Original.flags'):
    flagmanager(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
      mode = 'save',
      versionname = 'Original')
  


# Initial flagging
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  # Flagging shadowed data
  
  flagdata(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
    mode = 'shadow',
           flagbackup = False)
  
  # Flagging edge channels for TDM data 
  
  flagdata(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
    mode = 'manual',
    spw = '0:0~7;120~127,1:0~7;120~127,2:0~7;120~127,3:0~7;120~127,4:0~7;120~127,5:0~7;120~127,6:0~7;120~127,7:0~7;120~127,8:0~7;120~127,9:0~7;120~127,10:0~7;120~127,11:0~7;120~127', 
           flagbackup = False)
  



 
# Putting a model for the flux calibrator(s)
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

 
      
  # Latest extraction from up-to-date utilities July 2019  
  setjy(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
        standard = 'manual',
        field = 'J0522-3627',  
        fluxdensity = [3.403576, 0, 0, 0],
        spix = -0.257996,
        reffreq = '666.911575GHz',
        spw = '0,1,2,3,4,5,6,7')  # required due to dual SPW in B2B

  # Flux uncertainty +- 0.131773 Jy
  # meanAge = 0.0 


# Save flags before bandpass cal
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  
  # Extra note:
  # This is the minimal flagging required

  flagdata(vis='uid___A002_Xc660ef_X1b89.ms.split' , mode='quack', spw='8,9,10,11', field='0', quackinterval=2, quackmode='beg', flagbackup=False, scan='128', antenna='DA41,DA56,DA58,DA62,DV06,DV19,DV25')

  flagdata(vis='uid___A002_Xc660ef_X1b89.ms.split' , mode='manual', field='0', flagbackup=False, antenna='DA45', scan='9,10,11,12') # HF and LF 

  flagdata(vis='uid___A002_Xc660ef_X1b89.ms.split' , mode='manual', field='2', flagbackup=False, spw='8,9,10,11', timerange='06:00:18.2~06:00:18.5')

  flagdata(vis='uid___A002_Xc660ef_X1b89.ms.split' , mode='manual', field='2', flagbackup=False, spw='8,9,10,11', timerange='06:03:04.6~06:03:04.7')

  flagdata(vis='uid___A002_Xc660ef_X1b89.ms.split' , mode='manual',  flagbackup=False, timerange='06:14:10.0~06:18:20.0', antenna='DA50')

  flagdata(vis='uid___A002_Xc660ef_X1b89.ms.split' , mode='manual',  flagbackup=False, antenna='DV14,DV25,DA63,DV04,DV10,DV12')
  ## Due to slopes in the residual, i.e. LF to HF does not work due to system introduced gradient
   #DV13, PM03

  flagdata(vis='uid___A002_Xc660ef_X1b89.ms.split' , mode='manual',  flagbackup=False, timerange='06:04:47.3~06:04:47.6', field='4')

  flagdata(vis='uid___A002_Xc660ef_X1b89.ms.split' , mode='manual',  flagbackup=False, scan='6~12') 

  # in the end there is not enough good in this scan sequence of the DGC, calibration appears better using offset from the latter sequence


# Bandpass calibration   
mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  ## Extra note:
  ## The DGC source is also used as the bandpass. It was observed at the HF as a dedicated 
  ## bandpass source, but at the low frequency the DGC scans must be used as there is no
  ## dedicated LF bandpass target 

  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.split.ap_pre_bandpass') 
  gaincal(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
          caltable = 'uid___A002_Xc660ef_X1b89.ms.split.ap_pre_bandpass',
          field = '0',
          spw = '0,1,2,3,4,5,6,7,8,9,10,11',   # using all spws
          solint = 'int',
          minsnr=3.0,   
          refant = 'DV07',
          calmode = 'p')
  
  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X1b89.ms.split.ap_pre_bandpass', msName='uid___A002_Xc660ef_X1b89.ms.split', interactive=False)


  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.split.bandpass') 
  bandpass(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
           caltable = 'uid___A002_Xc660ef_X1b89.ms.split.bandpass',
           field = '0',
           solint = 'inf', # no freq binning TDM mode
           combine = 'scan', 
           refant = 'DV03',
           solnorm = True,
           bandtype = 'B',
           minsnr= 3.0, 
           gaintable = 'uid___A002_Xc660ef_X1b89.ms.split.ap_pre_bandpass')
		    
  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X1b89.ms.split.bandpass', msName='uid___A002_Xc660ef_X1b89.ms.split', interactive=False)




# Save flags before gain cal
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis ='uid___A002_Xc660ef_X1b89.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')





# Gain calibration DGC 
mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  # This is the DGC step - this is the unique step for the B2B modes
  # This involves solving for the LF to HF instrumental offset using the DiffGainCal (DGC)
  # source. Ultimately there are two ways to do this:

  # A LF to HF cross apply followed by the  offset solution (cross apply method)
  # This EXPLICITLY solves the DGC offset and is a solution to be applied
  # to the HF data in the final applycal stages  - THIS IS WHAT IS USED FOR THESE DATA 
  # AS THE MOST ROBUST B2B CALIBRATION TECHNIQUE FOR LB

  #### more infomation on the phaseDiff method (NOT USED) ####

  # The phaseDiff method - as each DGC 'sequence' is an observation for a short total time (<few mins)
  # but cycled fast (<30-40s between the HF and LF) to produce interleaved observations 
  # of the same target but alternate frequencies, then _IF_ the cycling is
  # fast enough to assume 'pairs' of the HF and LF sub-scans are really looking through the
  # 'same' atmopshere, then one can employ a simple phaseDiff solution at 
  # both the HF and the LF, i.e. single solutions for the HF and LF independently
  # aligning all SPW. For B2B in all calibration or application the HF or the LF
  # phaseDiff solutions must be applied as the assumption is that the B2B offset
  # is not dierectly solved, but _IMPLICIT_ from all SPW being alligned at a time 'T'
  # which is the mid-time point of the DGC used for the phaseDiff solutions

  # In practice, the phaseDiff and cross apply are equivalent 
  # however the phaseDiff can be senstive to poorer atmospheric conditions, whereby
  # many DGC sequences are used, due to a separation in time of more than 10s of 
  # miuntes, such that it _IS_ possible for the HF to phase wrap, while the corresponding
  # LF does not wrap. This meeans the phaseDiff solutions do not produce 
  # and implicit correction and there is a resulting offset after the application.
  # This can be solved then with another DGC HF offset solve, which again is an 
  # _EXPLICIT_ correction for the offset. This is avoidable if using only ONE DGC sequence
  # lasting ~5-6minutes, where the phase should not wrap - if it does then likely the phase
  # stability is so bad, that you shouldn't be observing !



  # Applying a phasediff to the LF as this gives us better SNR 

  # NOTE the DGC solution was tested iteratively and uses the latter scan group only (not the others are flagged above)

  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.split.DGCphaselow_inf_phasediff') 

  gaincal(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
    caltable = 'uid___A002_Xc660ef_X1b89.ms.split.DGCphaselow_inf_phasediff',
    field = '0',    
    scan='6,8,10,12,122,124,126,128',  # select all DGC LF scans (recall 6~16 flagged but not a problem to leave here)
    solint = 'inf',  # inf as we interpolate the solution
    refant = 'DV07',
    gaintype = 'G',
    calmode = 'p',
    minsnr=3.0,
    spw='8,9,10,11',
    combine='scan', 
    interp=['linear'],
    gaintable = ['uid___A002_Xc660ef_X1b89.ms.split.bandpass'],  # no phase diff 
    gainfield = ['0'])

  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X1b89.ms.split.DGCphaselow_inf_phasediff', msName='uid___A002_Xc660ef_X1b89.ms.split', interactive=False) 



  ## Solve the  LF phases in time (combine with SPW)

  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.split.DGCphaselow_inf') 

  gaincal(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
    caltable = 'uid___A002_Xc660ef_X1b89.ms.split.DGCphaselow_inf',
    field = '0',    
    scan='6,8,10,12,122,124,126,128',  # select all DGC LF scans (recall 6~16 flagged but not a problem to leave here)
    solint = 'inf',  # inf as we interpolate the solution
    refant = 'DV07',
    gaintype = 'G',
    calmode = 'p',
    minsnr=3.0,
    spw='8,9,10,11',
    combine='spw', 
    interp=['linear','linear'],
    gaintable = ['uid___A002_Xc660ef_X1b89.ms.split.bandpass','uid___A002_Xc660ef_X1b89.ms.split.DGCphaselow_inf_phasediff'],  # with phase diff 
    gainfield = ['0','0'])

  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X1b89.ms.split.DGCphaselow_inf', msName='uid___A002_Xc660ef_X1b89.ms.split', interactive=False) 




  ## apply to HF cross apply and solve the offset - for each HF SPW _alone_
  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.split.DGCoffset_inf') 
  gaincal(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
    caltable = 'uid___A002_Xc660ef_X1b89.ms.split.DGCoffset_inf',
    field = '0',  #
    scan='7,9,11,123,125,127', # select all DGC HF scans (recall 7~15 flagged but not a problem to leave here)
    spw='0,1,2,3,4,5,6,7',
    solint = 'inf',  
    refant = 'DV07',
    gaintype = 'G',
    calmode = 'p',
    minsnr=3.0,
    combine='scan', # average to make ONE offset solution in this case 
    interp=['linear','linearPD'], 
    spwmap=[[],[8,8,8,8,8,8,8,8]], # in order of best matching lowest with lowest, highest with highest SPW pairs 
    gainfield = ['0','0'],  
    gaintable = ['uid___A002_Xc660ef_X1b89.ms.split.bandpass','uid___A002_Xc660ef_X1b89.ms.split.DGCphaselow_inf'])  # no phase diff 

  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X1b89.ms.split.DGCoffset_inf', msName='uid___A002_Xc660ef_X1b89.ms.split', interactive=False)


  ## TESTING PLOTS - these are residuals to solve the HF data after
  ## the LF and DGC has been applied - these should be close to zero (but can have noise)
  ## NOTE - the DGC scan was not ideal as part of these tests 

  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.split.DGCresidual') 
  gaincal(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
    caltable = 'uid___A002_Xc660ef_X1b89.ms.split.DGCresidual',
    field = '0',  
    scan='7,9,11,123,125,127', # select all DGC HF scans (recall 7~15 flagged but not a problem to leave here)
    spw='0,1,2,3,4,5,6,7',
    solint = 'int',  ## so we can see the noise  
    refant = 'DV07',
    gaintype = 'G',
    calmode = 'p',
    minsnr=3.0,
    refantmode='strict',  # need to check if the solutions are flat and at zero as this is a diagnostic plot
    interp=['linear','linearPD','linear'], 
    spwmap=[[],[8,8,8,8,8,8,8,8],[]], 
    gainfield = ['0','0','0'],  
    gaintable = ['uid___A002_Xc660ef_X1b89.ms.split.bandpass','uid___A002_Xc660ef_X1b89.ms.split.DGCphaselow_inf','uid___A002_Xc660ef_X1b89.ms.split.DGCoffset_inf'])


  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X1b89.ms.split.DGCresidual', msName='uid___A002_Xc660ef_X1b89.ms.split', interactive=False)

  ## NOTE most of these are on average about zero deg phase, meaning the LF transfer and the DGC offset corrected them reasonably
  ## note due to testing the cycling on the DGC, LF to HF was slower than required and only having 3 HF
  ## repeats, which is not the best for DGC offsetting - long term selfcal if possible might help 
  ## the data - this is not performed in this script 


# Gain calibration phase cal step and amplitude calibration 
mystep = 16
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  # Only need the INF solution for the LF as they are always cross applied, never applied to self

  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.split.phaselow_inf') 
  gaincal(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
          caltable = 'uid___A002_Xc660ef_X1b89.ms.split.phaselow_inf', 
          field = '0,2', # Bandpass/DGC, and Phasecal    
          spw='8,9,10,11',
          solint = 'inf',  
          refant = 'DV07',
          combine='spw',
          gaintype = 'G',
          calmode = 'p',
          minsnr=3.0,
          append = True,
          gaintable = ['uid___A002_Xc660ef_X1b89.ms.split.bandpass','uid___A002_Xc660ef_X1b89.ms.split.DGCphaselow_inf_phasediff'], # with phase diff 
          gainfield = ['0','0'])

  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X1b89.ms.split.phaselow_inf', msName='uid___A002_Xc660ef_X1b89.ms.split', interactive=False)



  # Now the HF using the BP scan for phase up to set the flux scale
  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.split.phasehigh_int') 
  gaincal(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
          caltable = 'uid___A002_Xc660ef_X1b89.ms.split.phasehigh_int', 
          field = '0',
          scan = '3',
          spw='0,1,2,3,4,5,6,7',
          solint = 'int',  
          refant = 'DV07',
          gaintype = 'G',
          calmode = 'p',
          minsnr=3.0,
          gaintable = 'uid___A002_Xc660ef_X1b89.ms.split.bandpass',
          gainfield = '0')

  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X1b89.ms.split.phasehigh_int', msName='uid___A002_Xc660ef_X1b89.ms.split', interactive=False)



  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.split.flux_inf*') 
  gaincal(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
          caltable = 'uid___A002_Xc660ef_X1b89.ms.split.flux_inf',  
          field = '0',
          spw='0,1,2,3,4,5,6,7',
          solint = 'inf',  
          #combine='scan',  
          scan = '3', # only BP segment
          refant = 'DV07',
          gaintype = 'T',
          calmode = 'a',
          minsnr=1.0,
          gaintable = ['uid___A002_Xc660ef_X1b89.ms.split.bandpass','uid___A002_Xc660ef_X1b89.ms.split.phasehigh_int'],
          gainfield = ['0','0'])


  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X1b89.ms.split.flux_inf', msName='uid___A002_Xc660ef_X1b89.ms.split', interactive=False)



# Save flags before applycal
mystep = 17
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')





# Application of gain tables 
mystep = 18
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  # Extra notes:
  # We never calibrate the HF data with 'itself' for the HF data, we always use the 
  # cross apply. In these data it is a 'unique' case that the BP and DGC are the same
  # this is not a necessity. However it allows us also an additional test
  # to see how well LF to HF transfer is, for application to the source itself
  # as we know its flux, and it is a strong point source 


  # Bandpass/DGC apply LF

  applycal(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
    field = '0', 
    spw='0,1,2,3,4,5,6,7', 
    interp=['linear','linearPD', 'linear','linear'],
    gaintable = ['uid___A002_Xc660ef_X1b89.ms.split.bandpass','uid___A002_Xc660ef_X1b89.ms.split.phaselow_inf','uid___A002_Xc660ef_X1b89.ms.split.DGCoffset_inf','uid___A002_Xc660ef_X1b89.ms.split.flux_inf'],
           spwmap = [[],[8,8,8,8,8,8,8,8],[],[]],
           gainfield = ['0','0','0','0'])


  # Now the target HL Tau (and the check source with the LF transfer
  # Extra note - the check source ....

  applycal(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
    field = '3,4',
    spw='0,1,2,3,4,5,6,7', 
    interp=['linear','linearPD', 'linear','linear'],
    gaintable = ['uid___A002_Xc660ef_X1b89.ms.split.bandpass','uid___A002_Xc660ef_X1b89.ms.split.phaselow_inf','uid___A002_Xc660ef_X1b89.ms.split.DGCoffset_inf','uid___A002_Xc660ef_X1b89.ms.split.flux_inf'],
           spwmap = [[],[8,8,8,8,8,8,8,8],[],[]],
    gainfield = ['0','2','0','0'])  # only the phase cal to map, rest is from BP or DGC (same source here)



# Split out corrected column 
mystep= 19
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  # Split out the full dataset

  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.split.cal') 
  os.system('rm -rf uid___A002_Xc660ef_X1b89.ms.split.cal.flagversions') 
  
  
  split(vis = 'uid___A002_Xc660ef_X1b89.ms.split',
    outputvis = 'uid___A002_Xc660ef_X1b89.ms.split.cal',
    datacolumn = 'corrected',
    keepflags = True)
  

# Save flags after applycal
mystep = 20
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xc660ef_X1b89.ms.split.cal',
    mode = 'save',
    versionname = 'AfterApplycal')
	  
